import base64
import json
import re
import bs4
from ..Empty import Album, Singer, Song, BaseAlbum, SearchMusic, MusicInfo, Lyric
from ..Empty import ResultSearchMusic, ResultAlbum, ResultMusicInfo
from ..Utils import net_get as _g, net2_session

_MUSIC_QQ_DOWNLOAD_HEADERS = {
    # ":authority": "u.y.qq.com",
    # ":method": "GET",
}
_MUSIC_QQ_SEARCH_HEADERS = {
    "Accept": "application/json,text/plain,*/*",
    "Accept-Encoding": "gzip,deflate",
    "Accept-Language": "zh-CN",
    "Connection": "keep-alive",
    "Host": "i.y.qq.com",
    "Origin": "http://y.qq.com/",
    "Referer": "http://y.qq.com/",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.119 Safari/537.36"
}
_MUSIC_QQ_SEARCH_PARAMS = {
    "g_tk": "938407465",
    "uin": "0",
    "format": "jsonp",
    "inCharset": "utf-8",
    "outCharset": "utf-8",
    "notice": "0",
    "platform": "h5",
    "needNewCode": "1",
    "zhidaqu": "1",
    "catZhida": "1",
    "t": "0",
    "flag": "1",
    "ie": "utf-8",
    "sem": "1",
    "aggr": "0",
    "remoteplace": "txt.mqq.all",
    "jsonpCallback": "json",
}
_MUSIC_QQ_DOWNLOAD_LYRIC_HEADERS = {
    "Accept": "application/json, text/plain, */*",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN",
    "Connection": "keep-alive",
    "Host": "i.y.qq.com",
    "Origin": "http://y.qq.com/",
    "Referer": "http://y.qq.com/",
    "from": "webapp_music",
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-origin",
    "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 13_2_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.0.3 Mobile/15E148 Safari/604.1",
}
_MUSIC_QQ_DOWNLOAD_LYRIC_PARAMS = {
    "songmid": "",
    "loginUin": "0",
    "hostUin": "0",
    "format": "jsonp",
    "inCharset": "GB2312",
    "outCharset": "utf-8",
    "notice": "0",
    "platform": "yqq",
    "jsonpCallback": "MusicJsonCallback",
    "needNewCode": "0",
}


def DEF_MUSIC_QQ_DOWNLOAD_DATA(id):
    return {
        "loginUin": 0,
        "hostUin": 0,
        "format": "json",
        "inCharset": "utf8",
        "outCharset": "utf-8",
        "notice": 0,
        "platform": "yqq.json",
        "needNewCode": 0,
        "data": json.dumps({
            "req_0": {
                "module": "vkey.GetVkeyServer", "method": "CgiGetVkey",
                "param": {
                    "guid": "10000",
                    "songmid": [id],
                    "songtype": [0],
                    "uin": "0",
                    "loginflag": 1,
                    "platform": "20"}},
            "comm": {
                "uin": 0,
                "format": "json",
                "ct": 20,
                "cv": 0
            }
        }, ensure_ascii=False)
    }


class QQ(object):
    def __init__(self):
        self.session = net2_session(["https://y.qq.com/",])

    def search(self, _text, page=1, limit=20) -> ResultSearchMusic:
        _MUSIC_QQ_SEARCH_PARAMS.update({
            "perpage": limit,
            "n": limit,
            "p": page,
            "w": _text,
        })
        try:
            res = re.findall(r'^json\((.*?)\)$',
                             _g("http://i.y.qq.com/s.music/fcgi-bin/search_for_qq_cp",
                                data=_MUSIC_QQ_SEARCH_PARAMS,
                                headers=_MUSIC_QQ_SEARCH_HEADERS).text)
            res = json.loads(res[0])
            songs = res.get("data", {}).get("song", {}).get("list", [])
            _data = SearchMusic({
                "songs": [Song({
                    "id": _.get("songmid"),
                    "name": _.get("songname"),
                    "duration": _.get("interval", 0),
                    "subtitle": _.get("songname_hilight", ""),
                    "album": BaseAlbum({
                        "id": _.get("albummid"),
                        "name": _.get("albumname"),
                    }),
                    "singers": [Singer({
                        "id": __.get("mid"),
                        "name": __.get("name"),
                    }) for __ in _.get("singer")],
                }) for _ in songs],
                "page": page,
                "limit": limit,
                "songCount": res.get("data", {}).get("song", {}).get("totalnum", 0)
            })
            return ResultSearchMusic(_data)
        except:
            return ResultSearchMusic(None, False)

    def album(self, albumid) -> ResultAlbum:
        try:
            content = _g(
                "https://i.y.qq.com/n2/m/share/details/album.html?ADTAG=newyqq.album&source=ydetail&albummid=" + albumid).text
            body = self.session.get("https://y.qq.com/n/yqq/album/" + albumid + ".html").text
            res = re.findall(r'<script>var firstPageData = (.*?)</script>\n', content)
            res = eval(res[0])
            imgs = bs4.BeautifulSoup(body, "html.parser").select("#albumImg")
            img = imgs[0]['src'] if len(imgs) else ""
            albumData = res.get("albumData", {})
            _data = Album({
                "id": albumData.get("albumMid"),
                "name": albumData.get("albumName"),
                "publishDate": albumData.get("publishDate"),
                "description": albumData.get("desc"),
                "pics": [
                    img,
                ],
                "singers": [Singer({
                    "id": albumData.get("singer", {}).get("mid", ""),
                    "name": albumData.get("singer", {}).get("name", ""),
                })],
                "songs": [Song({
                    "id": _.get("songInfo").get("mid"),
                    "name": _.get("songInfo").get("name"),
                    "subtitle": _.get("songInfo").get("title"),
                    "duration": _.get("interval", 0),
                    "singers": [Singer({
                        "id": __.get("mid"),
                        "name": __.get("name"),
                    }) for __ in _.get("songInfo", {}).get("singer", [])]
                }) for _ in albumData.get("list")],
            })
            return ResultAlbum(_data)
        except BaseException as base:
            return ResultAlbum(None, False)

    def detail(self, id) -> ResultMusicInfo:
        # songmid
        try:
            # 音乐源地址
            res = _g("https://u.y.qq.com/cgi-bin/musicu.fcg",
                                   data=DEF_MUSIC_QQ_DOWNLOAD_DATA(id),
                                   headers=_MUSIC_QQ_DOWNLOAD_HEADERS)
            res = json.loads(res.text).get("req_0", {}).get("data", {})
            play_list = res.get("midurlinfo", [])

            mp3 = "http://ws.stream.qqmusic.qq.com/"
            if len(play_list) >= 1:
                if play_list[0].get("purl") != "":
                    mp3 +=play_list[0].get("purl", "")
                else:
                    mp3 +=res.get("testfilewifi", "")
            else:
                mp3 +=res.get("testfilewifi", "")
            # 歌词
            _MUSIC_QQ_DOWNLOAD_LYRIC_PARAMS.update({"songmid": id})
            res = re.findall(r'^MusicJsonCallback\((.*?)\)$',
                             _g("http://i.y.qq.com/lyric/fcgi-bin/fcg_query_lyric.fcg",
                                data=_MUSIC_QQ_DOWNLOAD_LYRIC_PARAMS,
                                headers=_MUSIC_QQ_DOWNLOAD_LYRIC_HEADERS).text)[0]
            res = eval(res)
            lrc = base64.b64decode(res.get("lyric")).decode()
            # 基础信息
            body = _g("https://y.qq.com/n/yqq/song/" + id + ".html").text
            _bs = bs4.BeautifulSoup(body, "html.parser")
            imgs = _bs.select(".data__photo")
            img = imgs[0]['src'] if len(imgs) else ""
            song_names = _bs.select(".data__name .data__name_txt")
            song_name = song_names[0].text if len(song_names) else ""
            singers = _bs.select(".data__singer a.data__singer_txt")

            _data = MusicInfo({
                "id": id,
                "name": song_name,
                "mp3_url": mp3,
                "music_url": res.get("data", {}).get("play_backup_url", None),
                "pics": [
                    img,
                ],
                "lyrics": [Lyric({"line": _[1],
                                  "time": _[0]}) for _ in
                           re.findall(r'\[([0-9]{2}:[0-9]{2}.[0-9]*?)\](.*?)\n', lrc)],
                "singers": [Singer({
                    "id": _["data-mid"],
                    "name": _.text,
                    "pics": [
                        img,
                    ]
                }) for _ in singers],
            })
            return ResultMusicInfo(_data)
        except BaseException as base:
            return ResultMusicInfo(None, False)



from .Site import KuGou
from .Site import KuWo
from .Site import MiGu
from .Site import NetEase
from .Site import QianQian
from .Site import QQ
from .Utils import prettytable
from .Utils import transform
from .Utils import request


import json


class BaseEmpty(object):
    def to_json(self) -> str:
        return json.dumps(self.__dict__, default=lambda o: {_: __ for _, __ in o.__dict__.items() if __ is not None},
                          ensure_ascii=False)

    def __str__(self):
        return self.__dict__.__str__()

    def to_dict(self) -> dict:
        return json.loads(
            json.dumps(self.__dict__, default=lambda o: {_: __ for _, __ in o.__dict__.items() if __ is not None},
                       ensure_ascii=False))



from ..Empty.BaseEmpty import BaseEmpty


class Lyric(BaseEmpty):
    def __init__(self, _dic=None):
        if _dic is None:
            _dic = {}
        self.line: str = _dic.get("line","")
        self.time: str = _dic.get("time","")


import json
import re
import time
import uuid
import bs4
from ..Empty import Album, Singer, Song, BaseAlbum, SearchMusic, MusicInfo, Lyric
from ..Empty import ResultSearchMusic, ResultAlbum, ResultMusicInfo
from ..Utils import net_get as _g, net2_session
from ..Utils import _MM_TO_MIN

_KUWO_SEARCH_HEADER = {
    "Origin": "search.kuwo.cn",
    "Referer": "search.kuwo.cn",
    "Host": "search.kuwo.cn",
    "Accept": "application/json, text/plain, */*",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8"
}


class KuWo(object):
    def __init__(self):
        self.session = net2_session(["https://www.kuwo.cn/", ])

    def search(self, _text, page=1, limit=20) -> ResultSearchMusic:
        _p = page - 1 if page >= 1 else page
        try:
            res = re.findall(r'^try\{var jsondata=(\{.*?\})\n;', _g("http://search.kuwo.cn/r.s", data={
                "all": _text,
                "pn": _p,
                "rn": limit,
                "httpsStatus": 1,
                "ft": "music",
                "client": "kt",
                "cluster": 0,
                "rformat": "json",
                "callback": "searchMusicResult",
                "encoding": "utf8",
                "r": int(time.time())
            }, headers=_KUWO_SEARCH_HEADER).text)
            res = eval(res[0])
            _data = SearchMusic({
                "songs": [Song({
                    "id": str(_.get("MUSICRID")).replace("MUSIC_", ""),
                    "name": str(_.get("SONGNAME")).replace("&nbsp;", " "),
                    "duration": _.get("DURATION", 0),
                    "album": BaseAlbum({
                        "id": _.get("ALBUMID"),
                        "name": _.get("ALBUM").replace("&nbsp;", " "),
                    }),
                    "singers": [Singer({
                        "id": _.get("ARTISTID"),
                        "name": str(_.get("ARTIST")).replace("&nbsp;", " "),
                    })],
                }) for _ in res.get("abslist", [])],
                "page": page,
                "limit": limit,
                "songCount": int(res.get("TOTAL", 0)),
            })
            return ResultSearchMusic(_data)
        except:
            return ResultSearchMusic(None, False)

    def album(self, albumid) -> ResultAlbum:
        try:
            res = _g("http://search.kuwo.cn/r.s", data={
                "pn": "0",
                "rn": "1000",
                "stype": "albuminfo",
                "albumid": albumid,
                "alflac": "1",
                "pcmp4": "1",
                "encoding": "utf8",
                "vipver": "MUSIC_8.7.7.0_W4"
            }, headers=_KUWO_SEARCH_HEADER).text
            res = eval(res)
            _data = Album({
                "id": res.get("albumid"),
                "name": res.get("name"),
                "publishDate": res.get("pub"),
                "description": res.get("info"),
                "pics": [
                    res.get("hts_img"),
                ],
                "singers": [Singer({
                    "id": res.get("artist"),
                    "name": res.get("name"),
                })],
                "songs": [Song({
                    "id": _.get("id"),
                    "name": _.get("name"),
                    "subtitle": _.get("subtitle"),
                    "singers": [Singer({
                        "id": res.get("artistid"),
                        "name": res.get("artist"),
                    })]
                }) for _ in res.get("musiclist")],
            })
            return ResultAlbum(_data)
        except BaseException as base:
            return ResultAlbum(None, state=False)

    def detail(self, id) -> ResultMusicInfo:
        try:
            # 获取音乐地址
            res = _g("http://player.kuwo.cn/webmusic/st/getNewMuiseByRid?rid=MUSIC_" + id)
            _xml = bs4.BeautifulSoup(res.text, "html.parser")
            # 获取歌词信息
            res = _g("http://m.kuwo.cn/newh5/singles/songinfoandlrc", data={
                "musicId": id,
                "httpsStatus": 1,
                "reqId": str(uuid.uuid4())
            }, headers={
                "Accept-Encoding": "gzip, deflate",
                "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
                "Host": "m.kuwo.cn",
            })
            res = json.loads(res.text)
            data = res.get("data", {})
            lrclist = data.get("lrclist", [])
            try:
                mp3 = _xml.select("mp3dl")[0].text + _xml.select("mp3path")[0].text
            except:
                mp3 = ""
            try:
                aac = _xml.select("aacdl")[0].text + _xml.select("aacpath")[0].text
            except:
                aac = ""
            _data = MusicInfo({
                "id": data.get("songinfo", {}).get("id", ""),
                "name": str(data.get("songinfo", {}).get("songName", "")).replace("&nbsp;", " "),
                "mp3_url": mp3,
                "music_url": aac,
                "pics": [
                    data.get("songinfo", {}).get("pic", None),
                ],
                "lyrics": [Lyric({"line": _.get("lineLyric", ""),
                                  "time": _MM_TO_MIN(_.get("time", ""))}) for _ in
                           lrclist],
                "singers": [Singer({
                    "id": data.get("songinfo", {}).get("artistId", ""),
                    "name": data.get("songinfo", {}).get("artist", ""),
                    "pics": [data.get("songinfo", {}).get("pic", "")]
                })],
            })
            return ResultMusicInfo(_data)
        except:
            return ResultMusicInfo(None, False)



import json
import re
import time
from ..Empty import Album, Singer, Song, BaseAlbum, SearchMusic, MusicInfo, Lyric
from ..Empty import ResultSearchMusic, ResultAlbum, ResultMusicInfo
from ..Utils import net2_session

_MUSIC_KG_SEARCH_PARAMS = {
    "callback": "callback123",
    "bitrate": "0",
    "isfuzzy": "0",
    "tag": "em",
    "inputtype": "0",
    "platform": "WebFilter",
    "userid": "-1",
    "clientver": "2000",
    "iscorrection": "1",
    "privilege_filter": "0",
    "srcappid": "2919",
    "clienttime": "1607052381011",
    "mid": "1607052381011",
    "uuid": "1607052381011",
    "dfid": "-",
    "signature": "2BD0F5EAD22C26C4FAF54ABEA736D1BD",
}
_MUSIC_KG_SEARCH_HEADERS = {
    "referer": "http://m.kugou.com",
    "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46"
                  + " (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1"}
_MUSIC_KG_DOWNLOAD_HEADERS = {
    ":authority": "wwwapi.kugou.com",
    ":method": "GET",
    ":scheme": "https",
}
_MUSIC_KG_ALBUM_HEADERS = {
    ":authority": "m3ws.kugou.com",
    ":method": "GET",
    ":scheme": "https",
    "accept": "*/*",
    "accept-encoding": "gzip, deflate, br",
    "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
    "referer": "https://m3ws.kugou.com/album/14456909.html",
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-origin",
    "user-agent": "Mozilla/5.0 (iPad; CPU OS 11_0 like Mac OS X) AppleWebKit/604.1.34 (KHTML, like Gecko) Version/11.0 Mobile/15A5341f Safari/604.1"
}


class KuGou(object):
    def __init__(self):
        self.session = net2_session(["http://m.kugou.com/", "https://m3ws.kugou.com/"])

    def search(self, _text, page=1, limit=30) -> ResultSearchMusic:
        # `ID ` use 'hash`
        try:
            _MUSIC_KG_SEARCH_PARAMS.update({
                "keyword": _text,
                "page": page,
                "pagesize": limit,
            })
            res = self.session.get("http://songsearch.kugou.com/song_search_v2",
                                   params={'keyword': _text,
                                           'platform': 'WebFilter',
                                           'format': 'json',
                                           'page': page,
                                           'pagesize': limit},
                                   headers=_MUSIC_KG_SEARCH_HEADERS).text
            res = eval(res)
            _data = SearchMusic({
                "songs": [Song({
                    "id": _.get("FileHash"),
                    "name": _.get("SongName"),
                    "duration": _.get("Duration", 0),
                    "subtitle": _.get("OriSongName", ""),
                    "album": BaseAlbum({
                        "id": _.get("AlbumID"),
                        "name": _.get("AlbumName"),
                    }),
                    "singers": [Singer({
                        "id": id,
                        "name": name,
                    }) for id, name in zip(_.get("SingerId"), str(_.get("SingerName", "")).split("、"))],
                }) for _ in res.get("data", {}).get("lists", [])],
                "page": page,
                "limit": limit,
                "songCount": res.get("data", {}).get("total", 0),
            })
            return ResultSearchMusic(_data)
        except:
            return ResultSearchMusic(None, False)

    def album(self, albumid) -> ResultAlbum:
        try:
            _MUSIC_KG_ALBUM_HEADERS.update({"referer": "https://m3ws.kugou.com/album/" + albumid + ".html"})
            res = self.session.get("https://m3ws.kugou.com/app/i/getablum.php", params={
                "type": 1,
                "ablumid": albumid
            }, headers=_MUSIC_KG_ALBUM_HEADERS).text
            res = eval(res)
            _data = Album({
                "id": albumid,
                "name": res.get("cname"),
                "publishDate": res.get("publish_date"),
                "description": res.get("intro"),
                "pics": [
                    res.get("img400"),
                    res.get("img"),
                ],
                "singers": [Singer({
                    "id": res.get("singerid"),
                    "name": res.get("singer"),
                })],
                "songs": [Song({
                    "id": _.get("hash", ""),
                    "name": _.get("songname"),
                    "subtitle": _.get("subtitle"),
                    "duration": _.get("Duration", 0),
                    "singers": [Singer({
                        "id": res.get("singerid"),
                        "name": res.get("singer"),
                    })]
                }) for _ in res.get("list")],
            })
            return ResultAlbum(_data)
        except BaseException as base:
            return ResultAlbum(None, state=False)

    def detail(self, hash, albumid) -> ResultMusicInfo:
        # MUSIC_XXXXXXXX hash
        try:
            # 获取音乐地址
            res = self.session.get("https://wwwapi.kugou.com/yy/index.php", params={
                "r": "play/getdata",
                "callback": "jQuery191030029284036250337_1607159310710",
                "hash": hash,
                "dfid": "",
                "mid": "00324ae2e621013f4c8c211113dd7016",
                "platid": "",
                "album_id": albumid,
                "_": int(time.time())
            })
            res = re.findall(r'^jQuery191030029284036250337_1607159310710\((.*?)\);$', res.text)
            res = json.loads(res[0])
            lrc = res.get("data", {}).get("lyrics", [])
            _data = MusicInfo({
                "id": hash,
                "name": res.get("data", {}).get("song_name", ""),
                "mp3_url": res.get("data", {}).get("play_url", ""),
                "music_url": res.get("data", {}).get("play_backup_url", None),
                "pics": [
                    res.get("data", {}).get("img", None),
                ],
                "lyrics": [Lyric({"line": _[1],
                                  "time": _[0]}) for _ in
                           re.findall(r'\[([0-9]{2}:[0-9]{2}.[0-9]*?)\](.*?)\n', lrc)],
                "singers": [Singer({
                    "id": _.get("author_id"),
                    "name": _.get("author_name"),
                    "pics":[_.get("avatar","")]
                }) for _ in res.get("data",{}).get("authors",[])],
            })
            return ResultMusicInfo(_data)
        except:
            return ResultMusicInfo(None, False)



from ..Empty.SongEmpty import  Song
from ..Empty.BaseEmpty import  BaseEmpty


class SearchMusic(BaseEmpty):
    def __init__(self, _dic=None):
        if _dic is None:
            _dic = {}
        self.songs: list[Song] = _dic.get("songs", [])
        self.page: int = _dic.get("page", 1)
        self.limit: int = _dic.get("limit", 20)
        self.songCount: int = _dic.get("songCount", 0)



from ..Empty.LyricEmpty import Lyric
from ..Empty.BaseEmpty import BaseEmpty
from ..Empty.SingerEmpty import Singer


class MusicInfo(BaseEmpty):
    def __init__(self, _dic=None):
        if _dic is None:
            _dic = {}
        self.id: str = _dic.get("id","")
        self.name: str = _dic.get("name","")
        self.mp3_url: str = _dic.get("mp3_url","")
        self.music_url: str = _dic.get("music_url","")
        self.lyrics: list[Lyric] = _dic.get("lyrics",[])
        self.pics: list[str] = _dic.get("pics", [])
        self.singers: list[Singer] = _dic.get("singers", [])


import getopt
import json
import os
import sys
import base64
from typing import Dict, Any
from .Utils import prettytable as pt, _STR_TO_FILE_NAME_
from .Utils import net_get as _g
from .Site import KuGou, QQ, QianQian, KuWo, NetEase, MiGu

code = {
    "v": "version",
    "s": "search",
    "S": "source",
    "d": "detail",
    "a": "album",
    "t": "type",
    "p": "page",
    "D": "download",
}
source: Dict[str, Any] = {
    "kg": KuGou(),
    "bd": QianQian(),
    "163": NetEase(),
    "qq": QQ(),
    "kw": KuWo(),
    "mg": MiGu(),
}


def id_decode(text):
    return base64.b64encode("|".join([str(text.get("id")),
                                      str(text.get("source")),
                                      str(text.get("album").get("id")),
                                      str(text.get("oid", ""))]).encode()).decode()


def id_encode(text):
    try:
        return (base64.b64decode(text).decode()).split("|")
    except:
        return []


def printFormat(datas, _TYPE, count=0, page=1):
    if _TYPE == "json":
        print(json.dumps({
            "count": count,
            "datas": [{
                "id": id_decode(_),
                "info": _
            } for _ in datas]
        }, ensure_ascii=True))
    elif _TYPE == "table":
        tb = pt.PrettyTable()
        tb.field_names = ["名称", "歌手", "时长", "id", "来源"]
        for _ in datas:
            tb.add_row([_.get("name"),
                        "/".join([__.get("name") for __ in _.get("singers")]),
                        _.get("duration"),
                        id_decode(_),
                        _.get("source")])
        if not count == 0:
            limit = len(datas)
            tb.add_row(["", "", "", "", ""])
            tb.add_row(["", "", "", "总量：%s" % count,
                        "%s/%s" % (str(page), str((count // limit) + (1 if count % limit > 0 else 0)))])
        print(tb)


def printFormatDetail(data, _TYPE):
    if _TYPE == "json":
        print(json.dumps(data, ensure_ascii=True))
    elif _TYPE == "table":
        tb = pt.PrettyTable()
        tb.title = "详细信息"
        tb.field_names = ["歌曲名", data.get("name")]
        tb.add_row(["歌手", "/".join([_.get("name") for _ in data.get("singers", [])])])
        print(tb)
        tb = pt.PrettyTable()
        tb.field_names = ["Time", "歌词"]
        for _ in data.get("lyrics"):
            tb.add_row([_.get("time"), str(_.get("line")).replace(" ", "").replace("\r", "")])
        print(tb)
        tb = pt.PrettyTable()
        tb.field_names = ["源地址"]
        tb.add_row([data.get("mp3_url")])
        print(tb)


def printFormatError(_TYPE):
    if _TYPE == "json":
        print(json.dumps({
            "code": 0
        }, ensure_ascii=True))
    elif _TYPE == "table":
        tb = pt.PrettyTable()
        tb.field_names = ["错误"]
        tb.add_row(["无法获取"])
        print(tb)


def main(**kwargs):
    dataType = ["json", "table"]
    _TYPE = "table"
    _SEARCH = None
    _CUR_CODE = None
    _SOURCE_CLS = None
    _SOURCE_CLS_NAME = None
    _VERSION = False
    _DETAIL = False
    _DETAIL_TEXT = None
    _PAGE = 1
    _DOWNLOAD = False
    _DOWNLOAD_PATH = os.path.expanduser('~') + "\\Music\\"
    _MUSIC_DATA = None
    _ALBUM = None
    argv = sys.argv[1:]
    _HELP_TEXT = pt.PrettyTable()
    _HELP_TEXT.field_names = ["option", "use", "arg"]
    _HELP_TEXT.add_row(["-v --version","", "查看版本信息"])
    _HELP_TEXT.add_row(["-h --help","", "查看帮助文本"])
    _HELP_TEXT.add_row(["-s --search=<text>","h-music -s xxx or h-music xxx", "搜索文本(选用)"])
    _HELP_TEXT.add_row(["-p --page=<number>","h-music -s xxx -S 163 page 1", "搜索分页\n(需要配合--source使用)"])
    _HELP_TEXT.add_row(["-d --detail=<id>","h-music -d <id>", "查看详细信息\n(包含艺术家、歌曲名称、歌词、源地址)"])
    _HELP_TEXT.add_row(["-a --album=<id>","h-music -a <id>", "查看专辑详细信息(仅支持json格式)"])
    _HELP_TEXT.add_row(["-t --type=<`json`,`table`>","h-music xxx -t json", "输出格式，支持`json`和`table`"])
    _HELP_TEXT.add_row(["-S --source=<source>","h-music xxx -s 163", "搜索源(`kg`,`bd`,`163`,`qq`,`kw`,`mg`)"])
    _HELP_TEXT.add_row(["-D --download","h-music -d <id> -D", "保存源文件，在--detail时生效"])
    _HELP_TEXT.add_row(["-P --download_path=<dir_path>","h-music -d <id> -D -P D:", "源文件保存路径\n(它可以包含--download但必须有参数)"])
    _HELP_TEXT.add_row(["","","示例：h-music alan_walker -S 163"])
    _HELP_TEXT.valign = "m"
    _HELP_TEXT.align = "l"

    #######################################
    len_code = len(argv)

    if len_code == 0 or (len_code == 1 and argv[0] in ['-h','--help']):
        print(_HELP_TEXT)
        sys.exit(0)
    else:
        if not argv[0] in "-s:-v:-a:-d:-S:-t:-D:-p:-h:".split(":"):
            _SEARCH = argv[0].replace("_", "")
            del argv[0]
    #######################################
    try:
        opts, args = getopt.getopt(argv, "-s:-v-a:-d:-S:-t:-P:-D-p:-h:",
                                   ["text", "version", "album=", "detail=", "source=", "type=", "download_path",
                                    "download", "page",
                                    "help"])
    except getopt.GetoptError:
        print(_HELP_TEXT)
        sys.exit(2)
    #######################################
    # print(opts)
    for opt, arg in opts:
        # print("opt:%s, arg:%s" % (opt, arg,))
        if opt == '-v':
            tb = pt.PrettyTable()
            tb.field_names = ["version", "h_music. 20201213"]
            tb.add_row(["******", "Howardyun.top"])
            tb.add_row(["              ", "请勿商用"])
            print(tb)
            sys.exit(0)
        elif opt in ("-s", "--text"):
            _SEARCH = arg.replace("_", "")
        elif opt in ("-p", "--page"):
            try:
                _PAGE = int(arg)
            except:
                print("`%s` is the wrong number type" % arg)
        elif opt in ("-t", "--type"):
            if not arg in dataType:
                print("data type '%s' does not exist" % arg)
                sys.exit(0)
            _TYPE = arg
        elif opt in ("-S", "--Source"):
            arg = arg.lower()
            if not arg in source.keys():
                print("source '%s' does not exist" % arg)
                sys.exit(0)
            _SOURCE_CLS = source.get(arg)
            _SOURCE_CLS_NAME = arg
        elif opt in ("-d", "--detail"):
            _DETAIL = True
            _DETAIL_TEXT = arg
        elif opt in ("-a", "--album"):
            _ALBUM = True
            _DETAIL_TEXT = arg
        elif opt in ("-D", "--download"):
            _DOWNLOAD = True
        elif opt in ("-P", "--download_path"):
            _DOWNLOAD = True
            if os.path.isdir(arg):
                _DOWNLOAD_PATH = arg
            else:
                print("path `%s` does not exist" % arg)
                sys.exit(0)
        else:
            print("command '%s' does not exist" % opt)

    if _SEARCH is not None:
        data = []
        if _SOURCE_CLS is not None:
            res = _SOURCE_CLS.search(_SEARCH, _PAGE)
            if res.state:
                for _ in res.data.to_dict().get("songs"):
                    _.update({"source": _SOURCE_CLS_NAME})
                    data.append(_)
                printFormat(data, _TYPE, res.data.songCount, res.data.page)
            else:
                printFormatError(_TYPE)

        else:
            for key, rs in source.items():
                res = rs.search(_SEARCH)
                if res.state:
                    for _ in res.data.to_dict().get("songs"):
                        _.update({"source": key})
                        data.append(_)
                else:
                    printFormatError(_TYPE)
            printFormat(data, _TYPE)


    elif _DETAIL and _DETAIL is not None:
        ss = id_encode(_DETAIL_TEXT)
        m = source.get(ss[1], None)
        if m is not None:
            if ss[1] in ["bd", "163", "qq", "kw"]:
                res = m.detail(ss[0])
            elif ss[1] in ["kg"]:
                res = m.detail(ss[0], ss[2])
            elif ss[1] in ["mg"]:
                res = m.detail(ss[0], ss[3])
            if res.state:
                printFormatDetail(res.data.to_dict(), _TYPE)
                _MUSIC_DATA = res.data.to_dict()
            else:
                printFormatError(_TYPE)
        else:
            print("Parsing error!")
            sys.exit(0)
    if _DOWNLOAD:
        if _MUSIC_DATA is not None:
            mp3_url = _MUSIC_DATA.get("mp3_url")
            if mp3_url is not None and not mp3_url[:4] == "http":
                mp3_url = "http://" + mp3_url
            content = _g(mp3_url).content
            _FILE_NAME = " ".join(
                [_MUSIC_DATA.get("name"), "-", "&".join([_.get("name") for _ in _MUSIC_DATA.get("singers", [])])])
            _FILE_NAME = _STR_TO_FILE_NAME_(_FILE_NAME)
            _SAVE_FILE_PATH = os.path.join(_DOWNLOAD_PATH, _FILE_NAME + ".mp3")
            with open(_SAVE_FILE_PATH, mode="wb") as wb:
                wb.write(content)
            print("Download -> " + _SAVE_FILE_PATH)
        else:
            print("Need the support of '-d'")
            print("as -d xxx -D")

    if _ALBUM:
        if (_DETAIL_TEXT):
            ss = id_encode(_DETAIL_TEXT)
            m = source.get(ss[1], None)
            if m is not None:
                print(json.dumps(m.album(ss[2]).to_dict(), ensure_ascii=True))
            else:
                print("Parsing error!")
                sys.exit(0)
        else:
            print("`DETAIL ID` not received")


if __name__ == '__main__':
    main()



from ..Empty.BaseAlbumEmpty import BaseAlbum
from ..Empty.SingerEmpty import Singer
from ..Empty.BaseEmpty import BaseEmpty

class Song(BaseEmpty):
    def __init__(self, _dic=None):
        if _dic is None:
            _dic = {}
        self.id: str = _dic.get("id","")
        self.oid: str = _dic.get("oid",None)
        self.name: str = _dic.get("name","")
        self.subtitle: str = _dic.get("subtitle","")
        self.duration: int = _dic.get("duration",0)
        self.album: BaseAlbum = _dic.get("album",None)
        self.privilege: bool = _dic.get("privilege",False)
        self.singers: list[Singer] = _dic.get("singers",[])



from ..Empty.BaseEmpty import BaseEmpty


class BaseAlbum(BaseEmpty):
    def __init__(self, _dic=None):
        if _dic is None:
            _dic = {}
        self.id = _dic.get("id", "")
        self.name = _dic.get("name", "")



from ..Empty.SongEmpty import Song
from ..Empty.BaseAlbumEmpty import BaseEmpty
from ..Empty.SingerEmpty import Singer


class Album(BaseEmpty):
    def __init__(self, _dic=None):
        if _dic is None:
            _dic = {}
        self.id = _dic.get("id", "")
        self.name = _dic.get("name", "")
        self.publishDate = _dic.get("publishDate", "")
        self.description = _dic.get("description", "")
        self.pics: list[str] = _dic.get("pics", [])
        self.songs: list[Song] = _dic.get("songs", [])
        self.singers: list[Singer] = _dic.get("singers", [])



from ..Empty.SearchEmpty import SearchMusic
from ..Empty.AlbumEmpty import Album
from ..Empty.BaseEmpty import BaseEmpty
from ..Empty.MusicInfoEmpty import MusicInfo


class Result(BaseEmpty):
    def __init__(self, data, state=True):
        self.state: bool = state
        self.data: object = data


class ResultSearchMusic(BaseEmpty):
    def __init__(self, data, state=True):
        self.state: bool = state
        self.data: SearchMusic = data


class ResultAlbum(BaseEmpty):
    def __init__(self, data, state=True):
        self.state: bool = state
        self.data: Album = data


class ResultMusicInfo(BaseEmpty):
    def __init__(self, data, state=True):
        self.state: bool = state
        self.data: MusicInfo = data



def _MM_TO_MIN(mm):
    mm = float(mm)
    hm = str(int(round(mm, 2) * 100 % 100))
    hm = "0" + hm if len(hm) == 1 else hm
    m, s = divmod(int(float(mm)), 60)
    h, m = divmod(m, 60)
    return "%s:%s.%s" % (str(m), str(s), str(hm))


def _MIN_TO_MM(min):
    "2:20"
    sp = str(min).split(":")
    if len(sp) == 3:
        hour, min, s = sp
        return int(hour) * 60 * 60 + int(min) * 60 + int(s)
    elif len(sp) == 2:
        min, s = sp
        return int(min) * 60 + int(s)


def _MIN_TO_MS(min):
    "2:20.11"
    sp = str(min).split(":")
    if len(sp) == 3:
        hour, min, s = sp
        s, ms = str(s).split(".")
        return int(hour) * 60 * 60 + int(min) * 60 + int(s) + float(ms) / 100
    elif len(sp) == 2:
        min, s = sp
        s, ms = str(s).split(".")
        return int(min) * 60 + int(s) + float(ms) / 100


def _STR_TO_FILE_NAME_(title):
    error_set = ['/', '\\', ':', '*', '?', '"', '|', '<', '>']
    for c in error_set:
        title = str(title).replace(c, "")
    return title



import requests
from hyper.contrib import HTTP20Adapter

def net_get(url, data=None, headers=None):
    if headers is None:
        headers = {}
    if data is None:
        data = {}
    return requests.get(url,params=data,headers=headers)

def net_post(url, data=None, headers=None):
    if headers is None:
        headers = {}
    if data is None:
        data = {}
    return requests.post(url,data=data,headers=headers)

def net2_session(index_url_list):
    session = requests.session()
    for index_url in index_url_list:
        session.mount(index_url, HTTP20Adapter())
    return session


import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

try:
    README = open(os.path.join(here, 'README.rst'), encoding='utf-8').read()
except:
    README = ""

setup(
    name='h_music',
    version='0.1.3',
    description='howard music',
    packages=find_packages('src'),
    package_dir={'': 'src'},

    # long_description="https://github.com/tt20050510/howard-music",
    long_description = README,
    url='https://github.com/tt20050510',
    author='HowardCheng',
    author_email='18071131140telephone@gmail.com',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='music spider setuptools development',
    install_requires=['requests', 'bs4'],
    extras_require={},
    package_data={},
    data_files=[],
    entry_points={'console_scripts': "h-music = h_music.__main__:main"},
    project_urls={
        'Bug Reports': 'https://github.com/tt20050510/howard-music/issues',
        'Source': "https://github.com/tt20050510/howard-music",
    },
)



import json
import re
import time
import bs4
from ..Empty import Album, Singer, Song, Result, BaseAlbum, SearchMusic, MusicInfo, Lyric
from ..Utils import net_get as _g, net2_session
from ..Utils import _MIN_TO_MM

_MUSIC_BD_SEARCH_HEADERS = {
    # ":authority": "music.taihe.com",
    # ":method": "GET",
    # ":path": "/search?word=%E8%BF%BD%E6%9C%88",
    # ":scheme": "https",
    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
    "accept-encoding": "gzip, deflate, br",
    "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
    "cache-control": "max-age=0",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36"
}


class QianQian(object):
    def __init__(self):
        self.session = net2_session(["https://music.taihe.com/"])

    def _get_detail(self, id):
        tracklink = _g("https://music.taihe.com/v1/song/tracklink", data={
            "TSID": id,
            "timestamp": int(time.time()),
            "from": "webapp_music",
            "s_protocol": "1"
        })
        tracklink = json.loads(tracklink.text)
        return tracklink

    def search(self, _text, page=1, limit=20) -> Result:
        _p = page - 1 if page >= 1 else page
        try:
            res = _g("https://music.taihe.com/search?word=" + _text, headers=_MUSIC_BD_SEARCH_HEADERS)
            _bs4 = bs4.BeautifulSoup(res.text, "html.parser")
            _songs = _bs4.select(".search-content li.clearfix")
            songs = [Song({
                "id": str(_.select(".song a")[0]["href"]).replace("/song/", ""),
                "name": str(_.select(".song a")[0].text),
                "duration": _MIN_TO_MM(_.select(".time")[0].text),
                "subtitle": None,
                "album": BaseAlbum({
                    "id": str(_.select(".album a")[0]["href"]).replace("/album/", ""),
                    "name": _.select(".album a")[0].text,
                }),
                "singers": [Singer({
                    "id": str(__["href"]).replace("/artist/", ""),
                    "name": str(__.text),
                }) for __ in _.select(".artist a")],
            }) for _ in _songs]
            _data = SearchMusic({
                "songs": songs,
                "page": page,
                "limit": limit,
                "songCount": len(songs),
            })
            return Result(_data)
        except BaseException as base:
            print(base)
            return Result(None, False)

    def album(self, albumid) -> Result:
        try:
            res = _g("https://music.taihe.com/album/" + albumid).text
            _bs4 = bs4.BeautifulSoup(res, "html.parser")
            _songs = _bs4.select(".page-container.page-main li.clearfix")
            publishDate = _bs4.select("div.other")
            description = _bs4.select("div.intro")
            try:
                pic = str(re.findall(r'pic:"(.*?)",type', res)[0]).replace("\\u002F", "\\")
            except:
                pic = ""
            _data = Album({
                "id": albumid,
                "name": str(_bs4.select("h1.title")[0].text),
                "publishDate": re.findall(r'[0-9]{4}-[0-9]{2}-[0-9]{2}', str(publishDate[0].text))[0] if len(
                    publishDate) >= 1 else "",
                "description": str(description[0].text) if len(description) >= 1 else "",
                "pics": [
                    pic,
                ],
                "singers": [{
                    "id": str(_["href"]).replace("/artist/", ""),
                    "name": str(_.text),
                } for _ in _bs4.select("span.artist a")],
                "songs": [Song({
                    "id": str(_.select(".song a")[0]["href"]).replace("/song/", ""),
                    "name": _.select(".song a")[0].text,
                    "subtitle": None,
                    "duration": _MIN_TO_MM(_.select(".time")[0].text),
                    "singer": [{
                        "id": str(_["href"]).replace("/artist/", ""),
                        "name": str(_.text),
                    } for _ in _.select(".artist a")]
                }) for _ in _songs],
            })
            return Result(_data)
        except:
            return Result(None, False)

    def detail(self, ids) -> Result:
        try:
            res = _g("https://music.taihe.com/v1/song/info",
                     data={"TSID": ids, "timestamp": int(time.time())})
            res = json.loads(res.text)
            res = res.get("data")[0]
            info = self._get_detail(res.get("assetId"))
            _data = MusicInfo({
                "id": res.get("assetId"),
                "name": info.get("data", {}).get("title", ""),
                "mp3_url": info.get("data", {}).get("path", ""),
                "music_url": None,
                "pics": [
                    info.get("data", {}).get("pic", "")
                ],
                "lyrics": [Lyric({"line": _[1],
                                  "time": _[0]}) for _ in
                           re.findall(r'\[([0-9]{2}:[0-9]{2}.[0-9]*?)\](.*?)\n', _g(res.get("lyric")).text)]
                if res.get("lyric") else res.get("lyric"),
                "singers": [Singer({
                    "id": _.get("artistCode"),
                    "name": _.get("name"),
                    "pics": [_.get("pic", "")]
                }) for _ in info.get("data", {}).get("artist", [])],
            })
            return Result(_data)
        except BaseException as base:
            # print(base)
            return Result(None, False)



from .AlbumEmpty import Album
from .ResultEmpty import Result,ResultSearchMusic, ResultAlbum, ResultMusicInfo
from .SearchEmpty import SearchMusic
from .SingerEmpty import Singer
from .SongEmpty import Song
from .BaseEmpty import BaseEmpty
from .BaseAlbumEmpty import BaseAlbum
from .MusicInfoEmpty import MusicInfo
from .LyricEmpty import Lyric


import json
import re
from ..Empty import Album, Singer, Song, BaseAlbum, SearchMusic, MusicInfo, Lyric
from ..Empty import ResultSearchMusic, ResultAlbum, ResultMusicInfo
from ..Utils import net_get as _g, net2_session

_MUSIC_MG_SEARCH_HEADERS = {
    "Accept": "application/json, text/javascript, */*; q=0.01",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Connection": "keep-alive",
    "Host": "m.music.migu.cn",
    "Sec-Fetch-Dest": "empty",
    "Sec-Fetch-Mode": "cors",
    "Sec-Fetch-Site": "same-origin",
    "User-Agent": "Mozilla/5.0 (iPad; CPU OS 11_0 like Mac OS X) AppleWebKit/604.1.34 (KHTML, like Gecko) Version/11.0 Mobile/15A5341f Safari/604.1",
    "X-Requested-With": "XMLHttpRequest",
    "Referer": "https://m.music.migu.cn/v3/search?page=1&type=song&i=eaaabf2729ba6f10feb3b3e92b6533d6888bd832&f=html&s=1607350181&c=001002A&keyword=alan&v=3.13.1",
    "Cookie": "JSESSIONID=58AA28AA568D3768A9BEC26ABC912D2B; mg_uem_user_id_3136357ddb6a49f5b317albumca6254e7ea49=bcb0e0f5-2f27-4408-aec4-e27bcdd22d9a; CookieID=YXCMND17B4XMVVT2EGAX6Z8DECB5VP8Q; migu_cookie_id=46c7fddb-4e41-45d9-8166-b06514409472-n41607335451399; migu_cn_cookie_id=b807a829-435a-4560-81a5-f7b270bf9dc3; Hm_lvt_ec5a5474d9d871cb3d82b846d861979d=1607350078; Hm_lpvt_ec5a5474d9d871cb3d82b846d861979d=1607350112; WT_FPC=id=2bf3b1d890e5436462e1607335273662:lv=1607350182649:ss=1607350065320"
}
_MUSIC_MG_ALL_HEADERS = {
    ":authority": "music.migu.cn",
    ":method": "GET",
    ":path": "/v3/search?page=1&type=song&i=06bb8afd7971a8ef4e8b5128581a9482cd6e9d38&f=html&s=1607352130&c=001002A&keyword=alan&v=3.13.1",
    ":scheme": "https",
    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
    "accept-encoding": "gzip, deflate, br",
    "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
    "cache-control": "max-age=0",
    "if-none-match": 'W/"a2f1-GJq8SeSfHs+T2WpvtzSWKjEoEoU',
    "referer": "https://music.migu.cn/v3",
    "sec-fetch-dest": "document",
    "sec-fetch-mode": "navigate",
    "sec-fetch-site": "same-origin",
    "sec-fetch-user": "?1",
    "upgrade-insecure-requests": "1",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36"
}


class MiGu(object):
    def __init__(self):
        self.session = net2_session(["https://music.migu.cn/"])

    def search(self, _text, page=1, limit=20) -> ResultSearchMusic:
        try:
            res = _g("https://m.music.migu.cn/migu/remoting/scr_search_tag"
                     "?rows=" + str(limit) + "&type=2&keyword=" + _text + "&pgc=" + str(page),
                     headers=_MUSIC_MG_SEARCH_HEADERS).text
            res = json.loads(res)
            _data = SearchMusic({
                "songs": [Song({
                    "id": _.get("id"),
                    "name": _.get("songName"),
                    "duration": 0,
                    "subtitle": _.get("title", ""),
                    "album": BaseAlbum({
                        "id": _.get("albumId"),
                        "name": _.get("albumName"),
                    }),
                    "singers": [Singer({
                        "id": id,
                        "name": name,
                    }) for id, name in
                        zip(str(_.get("singerId", "")).split(", "), str(_.get("singerName", "")).split(", "))],
                }) for _ in res.get("musics", [])],
                "page": page,
                "limit": limit,
                "songCount": res.get("data", {}).get("total", 0),
            })
            return ResultSearchMusic(_data)
        except BaseException as base:
            return ResultSearchMusic(None, False)

    def album(self, albumid) -> ResultAlbum:
        try:
            # Songs
            res = self.session.get(
                "https://m.music.migu.cn/migu/remoting/cms_album_song_list_tag?albumId=" + albumid + "&pageSize=1000").text
            songs = json.loads(res)
            # Album Info
            res = self.session.get("https://m.music.migu.cn/migu/remoting/cms_album_detail_tag?albumId=" + albumid).text
            res = json.loads(res)
            album = res.get("data", {})
            # Singer Info
            res = self.session.get(
                "https://m.music.migu.cn/migu/remoting/cms_artist_detail_tag?artistId=" + album.get("singerId")).text
            res = json.loads(res)
            singer = res.get("data", {})
            _data = Album({
                "id": albumid,
                "name": album.get("albumName"),
                "publishDate": album.get("publishDate"),
                "description": album.get("albumIntro"),
                "pics": [
                    album.get("albumPicL"),
                    album.get("albumPicM"),
                    album.get("albumPicS"),
                ],
                "singers": [Singer({
                    "id": singer.get("artistId"),
                    "name": singer.get("artistName"),
                    "pics": [
                        singer.get("artistPicL"),
                        singer.get("artistPicM"),
                        singer.get("artistPicS"),
                    ]
                })],
                "songs": [Song({
                    "id": _.get("songId"),
                    "name": _.get("songName"),
                    "duration": 0,
                    "oid": _.get("copyrightId"),
                    "subtitle": _.get("subtitle"),
                    "singers": [Singer({
                        "id": id,
                        "name": name,
                    }) for id, name in zip(_.get("singerId", []), _.get("singerName", []))]
                }) for _ in songs.get("result", {}).get("results", [])],
            })
            return ResultAlbum(_data)
        except BaseException as base:
            return ResultAlbum(None, False)

    def detail(self, id, oid = None) -> ResultMusicInfo:
        try:
            # 获取音乐地址
            res = _g("https://app.c.nf.migu.cn/MIGUM2.0/v2.0/content/listen-url", data={
                "netType": '01',
                "resourceType": 'E',
                "songId": id,
                "toneFlag": {
                    "128": 'PQ',
                    "320": 'HQ',
                    "flac": 'SQ',
                }["320"],
                "dataType": 2,
            }, headers={
                "referer": 'http://music.migu.cn/v3/music/player/audio',
                "channel": '0146951',
                "uid": "1234",
            })
            play = json.loads(res.text)
            lyrics = []
            if oid is not None:
                res = _g("https://m.music.migu.cn/migu/remoting/cms_detail_tag", data={"cpid": oid},
                         headers=_MUSIC_MG_SEARCH_HEADERS)
                res = json.loads(res.text)
                lrc = res.get("data", {}).get("lyricLrc", "")
                lyrics = [Lyric({"line": _[1],
                                      "time": _[0]}) for _ in
                               re.findall(r'\[([0-9]{2}:[0-9]{2}.[0-9]*?)\](.*?)\n', lrc)]
            _data = MusicInfo({
                "id": id,
                "name": play.get("data", {}).get("songItem", {}).get("songName"),
                "mp3_url": play.get("data", {}).get("url", ""),
                "music_url": None,
                "pics": [
                    _.get("img", "") for _ in play.get("data", {}).get("songItem", {}).get("albumImgs", [])
                ],
                "lyrics": lyrics,
                "singers": [Singer({
                    "id": _.get("id"),
                    "name": _.get("name"),
                    "pics": []
                }) for _ in play.get("data", {}).get("songItem", {}).get("artists", [])],
            })
            return ResultMusicInfo(_data)
        except BaseException as base:
            return ResultMusicInfo(None, False)



import base64
import binascii
import json
import os
import re
import time
import bs4
from Crypto.Cipher import AES

from ..Empty import Album, Singer, Song, BaseAlbum, SearchMusic, MusicInfo, Lyric
from ..Empty import ResultSearchMusic, ResultAlbum, ResultMusicInfo
from ..Utils.request import net_get as _g, net_post as _p, net2_session


class NetEase(object):
    def __init__(self):
        self.session = net2_session(["http://m.kugou.com/", "https://m3ws.kugou.com/"])

    def _create_key(self, size):
        return binascii.hexlify(os.urandom(size))[:16]

    def _aes(self, text, key):
        pad = 16 - len(text) % 16
        text = text + bytearray([pad] * pad)
        encryptor = AES.new(key, 2, b"0102030405060708")
        ciphertext = encryptor.encrypt(text)
        return base64.b64encode(ciphertext)

    def _rsa(self, text, pubkey, modulus):
        text = text[::-1]
        rs = pow(int(binascii.hexlify(text), 16), int(pubkey, 16), int(modulus, 16))
        return format(rs, "x").zfill(256)

    def _encrypted_request(self, data) -> dict:
        MODULUS = (
            "00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7"
            "b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280"
            "104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932"
            "575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b"
            "3ece0462db0a22b8e7"
        )
        PUBKEY = "010001"
        NONCE = b"0CoJUm6Qyw8W8jud"
        data = json.dumps(data).encode("utf-8")
        secret = self._create_key(16)
        params = self._aes(self._aes(data, NONCE), secret)
        encseckey = self._rsa(secret, PUBKEY, MODULUS)
        return {"params": params, "encSecKey": encseckey}

    def search(self, _text, page=1, limit=20) -> ResultSearchMusic:
        try:
            res = json.loads(_p("http://music.163.com/api/search/pc", data={
                'limit': limit,
                'offset': (page - 1) * limit,
                's': _text,
                'type': 1,
            }).text)
            _data = SearchMusic({
                "songs": [Song({
                    "id": _.get("id"),
                    "name": _.get("name"),
                    "duration": int(_.get("duration", 0) / 1000),
                    "subtitle": None,
                    "privilege": not (_.get("privilege", {}).get("fl", 0) == 0),
                    "album": BaseAlbum({
                        "id": _.get("album", {}).get("id"),
                        "name": _.get("album", {}).get("name"),
                    }),
                    "singers": [Singer({
                        "id": __.get("id"),
                        "name": __.get("name"),
                    }) for __ in _.get("artists", [])],
                }) for _ in res.get("result", {}).get("songs")],
                "page": page,
                "limit": limit,
                "songCount": res.get("result", {}).get("songCount")
            })
            return ResultSearchMusic(_data)
        except:
            return ResultSearchMusic(None, False)

    def album(self, albumid) -> ResultAlbum:
        try:
            res = _p("https://interface.music.163.com/weapi/v1/album/" + albumid, data=self._encrypted_request({}))
            res = json.loads(res.text)
            album = res.get("album", {})
            songs = [{
                "songid": _.get("id"),
                "songname": _.get("name"),
                "subtitle": _.get("name"),
            } for _ in res.get("songs")]
            _data = Album({
                "id": albumid,
                "name": album.get("name"),
                "publishDate": time.strftime("%Y-%m-%d", time.localtime(album.get("publishTime", 0) / 1000.0)),
                "description": album.get("description"),
                "pics": [
                    album.get("picUrl"),
                    album.get("blurPicUrl"),
                ],
                "singers": [Singer({
                    "id": __.get("id"),
                    "name": __.get("name"),
                }) for __ in album.get("artists", [])],
                "songs": [Song({
                    "id": _.get("id"),
                    "name": _.get("name"),
                    "subtitle": None,
                    "duration": _.get("Duration", 0),
                    "singers": [Singer({
                        "id": __.get("id"),
                        "name": __.get("name"),
                    }) for __ in _.get("ar", [])]
                }) for _ in res.get("songs")],
            })
            return ResultAlbum(_data)
        except:
            return ResultAlbum(None, False)

    def detail(self, id) -> ResultMusicInfo:
        try:
            # Play URL
            res = _p("http://music.163.com/weapi/song/enhance/player/url", data=self._encrypted_request({
                "ids": [id, ],
                "br": 32000
            }))
            play = json.loads(res.text).get("data")[0]
            # Lyric
            lrc = _p("https://music.163.com/weapi/song/lyric",
                     data=self._encrypted_request({"csrf_token": "", "id": id, "lv": -1, "tv": -1}))
            lrc = json.loads(lrc.text).get("lrc", {}).get("lyric", "")
            # INFO
            body = _g("https://music.163.com/song?id=" + id).text
            _bs = bs4.BeautifulSoup(body, "html.parser")
            imgs = _bs.select(".j-img")
            img = imgs[0]['src'] if len(imgs) else ""
            song_names = _bs.select(".tit .f-ff2")
            song_name = song_names[0].text if len(song_names) else ""
            singers = _bs.select(".des span .s-fc7")
            _data = MusicInfo({
                "id": play.get("id"),
                "name": song_name,
                "mp3_url": play.get("url", ""),
                "music_url": None,
                "pics": [
                    img,
                ],
                "lyrics": [Lyric({"line": _[1],
                                  "time": _[0]}) for _ in
                           re.findall(r'\[([0-9]{2}:[0-9]{2}.[0-9]*?)\](.*?)\n', lrc)],
                "singers": [Singer({
                    "id": str(_["href"]).replace("/artist?id=",""),
                    "name": _.text,
                    "pics": [
                        img,
                    ]
                }) for _ in singers]
            })
            return ResultMusicInfo(_data)
        except:
            return ResultMusicInfo(None, False)
